%%%============================================================================
%%% File     : eburg.hrl
%%% Purpose  : Record and macro definitions.
%%% Author   : j@svirfneblin.org
%%% License  : BSD
%%% Created  : 2007-07-11 14:45:31 j
%%% Modified : 2007-11-01 13:42:46 j
%%%============================================================================

%% Auto-import
-import(lists, [reverse/1, member/2, flatten/1, seq/2, duplicate/2, concat/1]).
-import(lists, [map/2, foldl/3, zip/2, foreach/2, all/2, any/2]).


-define(verbose(Expr),
        case get('$verbose') of
            true  -> Expr;
            false -> ok
        end).


%% The state record. Contains the internal representation of the input
%% specification plus some derived data structures and state variables.
-record(eburg,   {infile  = "",    % string
                  outfile = "",    % string
                  module  = "",    % string

                  ops,             % [atom]
                  nts,             % [atom]
                  nt_to_inn,       % [{atom, int}]
                  start,           % atom

                  rules,           % [#rule]
                  crules,          % [{atom, [#rule]}]
                  brules,          % [{atom, [#rule]}]

                  code}).          % [syntaxTree]


%% Rules are annotated with IRNs and line numbers.
-record(rule,    {number,          % int
                  line    = -1,    % int
                  lhs,             % atom
                  rhs,             % atom|#pattern
                  cost    = 0,     % int|string
                  action  = ""}).  % string


%% Patterns are used to represent the right-hand sides of base rules.
%% Nullary operators have an empty list of kids whereas nonterminals are
%% simply atoms.
-record(pattern, {op,              % atom
                  kids    = []}).  % [atom|#pattern]


%%% Local Variables:
%%% mode: erlang
%%% time-stamp-pattern: "Modified : %%$"
%%% End:
