%%%============================================================================
%%% File     : eburg.erl
%%% Purpose  : A code generator generator for Erlang.
%%% Author   : j@svirfneblin.org
%%% License  : BSD
%%% Created  : 2007-07-22 15:49:04 j
%%% Modified : 2007-11-01 13:35:38 j
%%%============================================================================

-module(eburg).
-author('j@svirfneblin.org').

-import(eburg_frontend, [read/2]).
-import(eburg_backend,  [emit/1]).

-export([file/1, file/2, main/2]).


%% @spec file(File::name()) -> ok | error
%% @equiv file(File, [])
file(File) ->
    file(File, []).


%% @spec file(File::name(), Opts::[term()]) -> ok | error
%% where
%%       name() = file:name()
%% @doc Returns `error' if there are errors or `ok' if there aren't.
%% `File' is the input specification. If there are no errors, the code
%% generator is written to "Base.erl" where `Base' is the basename of
%% `File'.
%%
%% Options:
%% <dl>
%%   <dt>`{outdir, string()}'</dt>
%%   <dt>`{verbose, bool()}'</dt>
%% </dl>
%% @end
%% Exceptions thrown by `eburg_util:die' and friends are caught here.
file(File, Opts) ->
    %cprof:stop(),
    %cprof:start(),
    Me = self(),
    spawn_opt(fun() -> Me ! (catch emit(read(File, Opts))) end,
              [{min_heap_size, 10000}]),
    receive
        Result ->
            %cprof:pause(),
            Result
    end.


%% @spec main(File::name(), Opts::[term()])
%% where
%%       name() = file:name()
%% @doc For command line use. Halts the Erlang runtime after calling `file'
%% and reports success or failure to the environment via its exit status.
main(File, Opts) ->
    case file(File, Opts) of
        ok    -> halt(0);
        error -> halt(1)
    end.


%%% Local Variables:
%%% mode: erlang
%%% time-stamp-pattern: "Modified : %%$"
%%% End:
