%% THIS IS A PRE-RELEASE OF LEEX - RELEASED ONLY BECAUSE MANY PEOPLE
%% WANTED IT - THE OFFICIAL RELEASE WILL PROVIDE A DIFFERENT INCOMPATIBLE
%% AND BETTER INTERFACE - BE WARNED
%% PLEASE REPORT ALL BUGS TO THE AUTHOR.

-module(eburg_lexer).

-export([string/1,string/2,token/2,token/3,tokens/2,tokens/3]).
-export([format_error/1]).

%% User code. This is placed here to allow extra attributes.
-import(eburg_util, [butlast/1]).
-export([lex/1]).

%% lex(File: string) -> string
%% lexes `File'. See EOF above.
lex(File) -> string(File ++ "$end").


stripc(Cost)   -> tl(tl(butlast(butlast(Cost)))).
stripa(Action) -> tl(butlast(butlast(Action))).

%%% Local Variables:
%%% mode: erlang
%%% time-stamp-pattern: "Modified : %%$"
%%% End:

format_error({illegal,S}) -> ["illegal characters ",io_lib:write_string(S)];
format_error({user,S}) -> S.

string(String) -> string(String, 1).

string(String, Line) -> string(String, Line, String, []).

%% string(InChars, Line, TokenChars, Tokens) ->
%%    {ok,Tokens,Line} | {error,ErrorInfo,Line}.

string([], L, [], Ts) ->			%No partial tokens!
    {ok,yyrev(Ts),L};
string(Ics0, L0, Tcs, Ts) ->
    case yystate(yystate(), Ics0, L0, 0, reject, 0) of
	{A,Alen,Ics1,L1} ->			%Accepting end state
	    string_cont(Ics1, L1, yyaction(A, Alen, Tcs, L1), Ts);
	{A,Alen,Ics1,L1,_S1} ->		%After an accepting state
	    string_cont(Ics1, L1, yyaction(A, Alen, Tcs, L1), Ts);
	{reject,_Alen,Tlen,_Ics1,L1,_S1} ->
	    {error,{L1,?MODULE,{illegal,yypre(Tcs, Tlen+1)}},L1};
	{A,Alen,_Tlen,_Ics1,L1,_S1} ->
	    string_cont(yysuf(Tcs, Alen), L1, yyaction(A, Alen, Tcs, L1), Ts)
    end.

%% string_cont(RestChars, Line, Token, Tokens)
%%  Test for and remove the end token wrapper.

string_cont(Rest, Line, {token,T}, Ts) ->
    string(Rest, Line, Rest, [T|Ts]);
string_cont(Rest, Line, {end_token,T}, Ts) ->
    string(Rest, Line, Rest, [T|Ts]);
string_cont(Rest, Line, skip_token, Ts) ->
    string(Rest, Line, Rest, Ts);
string_cont(_Rest, Line, {error,S}, _Ts) ->
    {error,{Line,?MODULE,{user,S}},Line}.

%% token(Continuation, Chars, Line) ->
%%    {more,Continuation} | {done,ReturnVal,RestChars}.
%% Must be careful when re-entering to append the latest characters to the
%% after characters in an accept.

token(Cont, Chars) -> token(Cont, Chars, 1).

token([], Chars, Line) ->
    token(Chars, Line, yystate(), Chars, 0, reject, 0);
token({Line,State,Tcs,Tlen,Action,Alen}, Chars, _) ->
    token(Chars, Line, State, Tcs ++ Chars, Tlen, Action, Alen).

%% token(InChars, Line, State, TokenChars, TokenLen, AcceptAction, AcceptLen) ->
%%    {more,Continuation} | {done,ReturnVal,RestChars}.

token(Ics0, L0, S0, Tcs, Tlen0, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
	{A1,Alen1,Ics1,L1} ->			%Accepting end state
	    token_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1));
	{A1,Alen1,[],L1,S1} ->			%After an accepting state
	    {more,{L1,S1,Tcs,Alen1,A1,Alen1}};
	{A1,Alen1,Ics1,L1,_S1} ->
	    token_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1));
	{A1,Alen1,Tlen1,[],L1,S1} ->		%After a non-accepting state
	    {more,{L1,S1,Tcs,Tlen1,A1,Alen1}};
	{reject,_Alen1,_Tlen1,eof,L1,_S1} ->
	    {done,{eof,L1},[]};
	{reject,_Alen1,Tlen1,Ics1,L1,_S1} ->
	    {done,{error,{L1,?MODULE,{illegal,yypre(Tcs, Tlen1+1)}},L1},Ics1};
	{A1,Alen1,_Tlen1,_Ics1,L1,_S1} ->
	    token_cont(yysuf(Tcs, Alen1), L1, yyaction(A1, Alen1, Tcs, L1))
    end.

%% tokens_cont(RestChars, Line, Token)
%%  If we have a token or error then return done, else if we have a
%%  skip_token then continue.

token_cont(Rest, Line, {token,T}) ->
    {done,{ok,T,Line},Rest};
token_cont(Rest, Line, {end_token,T}) ->
    {done,{ok,T,Line},Rest};
token_cont(Rest, Line, skip_token) ->
    token(Rest, Line, yystate(), Rest, 0, reject, 0);
token_cont(Rest, Line, {error,S}) ->
    {done,{error,{Line,?MODULE,{user,S}},Line},Rest}.

%% tokens(Continuation, Chars, Line) ->
%%    {more,Continuation} | {done,ReturnVal,RestChars}.
%% Must be careful when re-entering to append the latest characters to the
%% after characters in an accept.

tokens(Cont, Chars) -> tokens(Cont, Chars, 1).

tokens([], Chars, Line) ->
    tokens(Chars, Line, yystate(), Chars, 0, [], reject, 0);
tokens({tokens,Line,State,Tcs,Tlen,Ts,Action,Alen}, Chars, _) ->
    tokens(Chars, Line, State, Tcs ++ Chars, Tlen, Ts, Action, Alen);
tokens({skip_tokens,Line,State,Tcs,Tlen,Error,Action,Alen}, Chars, _) ->
    skip_tokens(Chars, Line, State, Tcs ++ Chars, Tlen, Error, Action, Alen).

%% tokens(InChars, Line, State, TokenChars, TokenLen, Tokens, AcceptAction, AcceptLen) ->
%%    {more,Continuation} | {done,ReturnVal,RestChars}.

tokens(Ics0, L0, S0, Tcs, Tlen0, Ts, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
	{A1,Alen1,Ics1,L1} ->			%Accepting end state
	    tokens_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1), Ts);
	{A1,Alen1,[],L1,S1} ->			%After an accepting state
	    {more,{tokens,L1,S1,Tcs,Alen1,Ts,A1,Alen1}};
	{A1,Alen1,Ics1,L1,_S1} ->
	    tokens_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1), Ts);
	{A1,Alen1,Tlen1,[],L1,S1} ->		%After a non-accepting state
	    {more,{tokens,L1,S1,Tcs,Tlen1,Ts,A1,Alen1}};
	{reject,_Alen1,_Tlen1,eof,L1,_S1} ->
	    {done,if Ts == [] -> {eof,L1};
		     true -> {ok,yyrev(Ts),L1} end,[]};
	{reject,_Alen1,Tlen1,_Ics1,L1,_S1} ->
	    skip_tokens(yysuf(Tcs, Tlen1+1), L1,
			{L1,?MODULE,{illegal,yypre(Tcs, Tlen1+1)}});
	{A1,Alen1,_Tlen1,_Ics1,L1,_S1} ->
	    tokens_cont(yysuf(Tcs, Alen1), L1, yyaction(A1, Alen1, Tcs, L1), Ts)
    end.

%% tokens_cont(RestChars, Line, Token, Tokens)
%%  If we have a end_token or error then return done, else if we have
%%  a token then save it and continue, else if we have a skip_token
%%  just continue.

tokens_cont(Rest, Line, {token,T}, Ts) ->
    tokens(Rest, Line, yystate(), Rest, 0, [T|Ts], reject, 0);
tokens_cont(Rest, Line, {end_token,T}, Ts) ->
    {done,{ok,yyrev(Ts, [T]),Line},Rest};
tokens_cont(Rest, Line, skip_token, Ts) ->
    tokens(Rest, Line, yystate(), Rest, 0, Ts, reject, 0);
tokens_cont(Rest, Line, {error,S}, _Ts) ->
    skip_tokens(Rest, Line, {Line,?MODULE,{user,S}}).

%%skip_tokens(InChars, Line, Error) -> {done,{error,Error,Line},Ics}.
%%  Skip tokens until an end token, junk everything and return the error.

skip_tokens(Ics, Line, Error) ->
    skip_tokens(Ics, Line, yystate(), Ics, 0, Error, reject, 0).

%% skip_tokens(InChars, Line, State, TokenChars, TokenLen, Tokens, AcceptAction, AcceptLen) ->
%%    {more,Continuation} | {done,ReturnVal,RestChars}.

skip_tokens(Ics0, L0, S0, Tcs, Tlen0, Error, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
	{A1,Alen1,Ics1,L1} ->			%Accepting end state
	    skip_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1), Error);
	{A1,Alen1,[],L1,S1} ->			%After an accepting state
	    {more,{skip_tokens,L1,S1,Tcs,Alen1,Error,A1,Alen1}};
	{A1,Alen1,Ics1,L1,_S1} ->
	    skip_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, L1), Error);
	{A1,Alen1,Tlen1,[],L1,S1} ->		%After a non-accepting state
	    {more,{skip_tokens,L1,S1,Tcs,Tlen1,Error,A1,Alen1}};
	{reject,_Alen1,_Tlen1,eof,L1,_S1} ->
	    {done,{error,Error,L1},[]};
	{reject,_Alen1,Tlen1,_Ics1,L1,_S1} ->
	    skip_tokens(yysuf(Tcs, Tlen1), L1, Error);
	{A1,Alen1,_Tlen1,_Ics1,L1,_S1} ->
	    skip_cont(yysuf(Tcs, Alen1), L1, yyaction(A1, Alen1, Tcs, L1), Error)
    end.

%% skip_cont(RestChars, Line, Token, Error)
%%  Skip tokens until we have an end_token or error then reutrn done
%%  with the original rror.

skip_cont(Rest, Line, {token,_T}, Error) ->
    skip_tokens(Rest, Line, yystate(), Rest, 0, Error, reject, 0);
skip_cont(Rest, Line, {end_token,_T}, Error) ->
    {done,{error,Error,Line},Rest};
skip_cont(Rest, Line, {error,_S}, Error) ->
    skip_tokens(Rest, Line, yystate(), Rest, 0, Error, reject, 0);
skip_cont(Rest, Line, skip_token, Error) ->
    skip_tokens(Rest, Line, yystate(), Rest, 0, Error, reject, 0).

yyrev(List) -> lists:reverse(List).
yyrev(List, Tail) -> lists:reverse(List, Tail).
yypre(List, N) -> lists:sublist(List, N).
yysuf(List, N) -> lists:nthtail(N, List).

%% yystate() -> InitialState.
%% yystate(State, InChars, Line, TokenLen, AcceptAction, AcceptLen) ->
%%      {Action, AcceptLength, RestChars, Line} |         Accepting end state
%%      {Action, AcceptLength, RestChars, Line, State} |  Accepting state
%%      {Action, AcceptLength, TokLength, RestChars, Line, State} |
%%      {reject, AcceptLength, TokLength, RestChars, Line, State}.
%% Generated state transition functions. The non-accepting end state
%% return signal either an unrecognised character or end of current
%% input.

yystate() -> 36.

yystate(39, [$a|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(37, Ics, Line, Tlen+1, _Action, _Alen);
yystate(39, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,39};
yystate(38, [$_|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(38, Ics, Line, Tlen+1, 2, Tlen);
yystate(38, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(38, Ics, Line, Tlen+1, 2, Tlen);
yystate(38, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $a, C =< $z ->
    yystate(38, Ics, Line, Tlen+1, 2, Tlen);
yystate(38, Ics, Line, Tlen, _Action, _Alen) ->
    {2,Tlen,Ics,Line,38};
yystate(37, [$r|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(33, Ics, Line, Tlen+1, _Action, _Alen);
yystate(37, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,37};
yystate(36, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(32, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(36, [$$|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(28, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$%|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(12, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$(|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(8, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$)|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(4, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$,|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(0, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$.|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(3, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$:|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$<|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(27, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$S|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(35, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$T|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(25, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [$[|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(14, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(32, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $\s ->
    yystate(32, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(11, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $A, C =< $R ->
    yystate(10, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $U, C =< $Z ->
    yystate(10, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $a, C =< $z ->
    yystate(38, Ics, Line, Tlen+1, _Action, _Alen);
yystate(36, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,36};
yystate(35, [$_|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(35, [$t|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(39, Ics, Line, Tlen+1, 3, Tlen);
yystate(35, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(35, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $A, C =< $Z ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(35, Ics, Line, Tlen, _Action, _Alen) ->
    {3,Tlen,Ics,Line,35};
yystate(34, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(34, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(34, [$:|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(18, Ics, Line, Tlen+1, _Action, _Alen);
yystate(34, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(34, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $9 ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(34, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $;, C =< $ ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(34, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,34};
yystate(33, [$t|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(29, Ics, Line, Tlen+1, _Action, _Alen);
yystate(33, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,33};
yystate(32, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(32, Ics, Line+1, Tlen+1, 12, Tlen);
yystate(32, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(32, Ics, Line, Tlen+1, 12, Tlen);
yystate(32, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $\s ->
    yystate(32, Ics, Line, Tlen+1, 12, Tlen);
yystate(32, Ics, Line, Tlen, _Action, _Alen) ->
    {12,Tlen,Ics,Line,32};
yystate(31, Ics, Line, Tlen, _Action, _Alen) ->
    {7,Tlen,Ics,Line};
yystate(30, [$]|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(26, Ics, Line, Tlen+1, _Action, _Alen);
yystate(30, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,30};
yystate(29, Ics, Line, Tlen, _Action, _Alen) ->
    {1,Tlen,Ics,Line};
yystate(28, [$e|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(24, Ics, Line, Tlen+1, _Action, _Alen);
yystate(28, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,28};
yystate(27, [$-|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(31, Ics, Line, Tlen+1, _Action, _Alen);
yystate(27, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,27};
yystate(26, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(34, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(26, [$:|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(18, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, [$]|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(30, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $9 ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $;, C =< $\\ ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $^, C =< $ ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(26, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,26};
yystate(25, [$_|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(25, [$e|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(21, Ics, Line, Tlen+1, 3, Tlen);
yystate(25, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(25, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $A, C =< $Z ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(25, Ics, Line, Tlen, _Action, _Alen) ->
    {3,Tlen,Ics,Line,25};
yystate(24, [$n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(20, Ics, Line, Tlen+1, _Action, _Alen);
yystate(24, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,24};
yystate(23, Ics, Line, Tlen, _Action, _Alen) ->
    {4,Tlen,Ics,Line};
yystate(22, [$]|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(26, Ics, Line, Tlen+1, 5, Tlen);
yystate(22, Ics, Line, Tlen, _Action, _Alen) ->
    {5,Tlen,Ics,Line,22};
yystate(21, [$r|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(17, Ics, Line, Tlen+1, _Action, _Alen);
yystate(21, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,21};
yystate(20, [$d|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(16, Ics, Line, Tlen+1, _Action, _Alen);
yystate(20, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,20};
yystate(19, [$\t|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(23, Ics, Line, Tlen+1, _Action, _Alen);
yystate(19, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(23, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(19, [$\s|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(23, Ics, Line, Tlen+1, _Action, _Alen);
yystate(19, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $\037 ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(19, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $!, C =< $ ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(19, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,19};
yystate(18, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(34, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(18, [$:|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(18, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, [$]|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(22, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $9 ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $;, C =< $\\ ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $^, C =< $ ->
    yystate(34, Ics, Line, Tlen+1, _Action, _Alen);
yystate(18, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,18};
yystate(17, [$m|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(13, Ics, Line, Tlen+1, _Action, _Alen);
yystate(17, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,17};
yystate(16, Ics, Line, Tlen, _Action, _Alen) ->
    {14,Tlen,Ics,Line};
yystate(15, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(15, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(15, [$.|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(19, Ics, Line, Tlen+1, _Action, _Alen);
yystate(15, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(15, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $- ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(15, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $/, C =< $ ->
    yystate(15, Ics, Line, Tlen+1, _Action, _Alen);
yystate(15, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,15};
yystate(14, [$:|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(26, Ics, Line, Tlen+1, _Action, _Alen);
yystate(14, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,14};
yystate(13, [$i|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(9, Ics, Line, Tlen+1, _Action, _Alen);
yystate(13, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,13};
yystate(12, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\000, C =< $\t ->
    yystate(12, Ics, Line, Tlen+1, 13, Tlen);
yystate(12, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $\v, C =< $ ->
    yystate(12, Ics, Line, Tlen+1, 13, Tlen);
yystate(12, Ics, Line, Tlen, _Action, _Alen) ->
    {13,Tlen,Ics,Line,12};
yystate(11, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(11, Ics, Line, Tlen+1, 6, Tlen);
yystate(11, Ics, Line, Tlen, _Action, _Alen) ->
    {6,Tlen,Ics,Line,11};
yystate(10, [$_|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(10, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $0, C =< $9 ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(10, [C|Ics], Line, Tlen, _Action, _Alen) when C >= $A, C =< $Z ->
    yystate(10, Ics, Line, Tlen+1, 3, Tlen);
yystate(10, Ics, Line, Tlen, _Action, _Alen) ->
    {3,Tlen,Ics,Line,10};
yystate(9, [$n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(5, Ics, Line, Tlen+1, _Action, _Alen);
yystate(9, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,9};
yystate(8, Ics, Line, Tlen, _Action, _Alen) ->
    {8,Tlen,Ics,Line};
yystate(7, Ics, Line, Tlen, _Action, _Alen) ->
    {11,Tlen,Ics,Line};
yystate(6, Ics, Line, Tlen, _Action, _Alen) ->
    {0,Tlen,Ics,Line};
yystate(5, [$a|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(1, Ics, Line, Tlen+1, _Action, _Alen);
yystate(5, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,5};
yystate(4, Ics, Line, Tlen, _Action, _Alen) ->
    {9,Tlen,Ics,Line};
yystate(3, [$\t|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(7, Ics, Line, Tlen+1, _Action, _Alen);
yystate(3, [$\n|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(7, Ics, Line+1, Tlen+1, _Action, _Alen);
yystate(3, [$\s|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(7, Ics, Line, Tlen+1, _Action, _Alen);
yystate(3, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,3};
yystate(2, [$s|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(6, Ics, Line, Tlen+1, _Action, _Alen);
yystate(2, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,2};
yystate(1, [$l|Ics], Line, Tlen, _Action, _Alen) ->
    yystate(2, Ics, Line, Tlen+1, _Action, _Alen);
yystate(1, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,1};
yystate(0, Ics, Line, Tlen, _Action, _Alen) ->
    {10,Tlen,Ics,Line};
yystate(S, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,S}.


%% yyaction(Action, TokenLength, TokenChars, TokenLine) ->
%%        {token,Token} | {end_token, Token} | skip_token | {error,String}.
%% Generated action function.

yyaction(0, _, _, TokenLine) ->
    {token,{'Terminals',TokenLine}};
yyaction(1, _, _, TokenLine) ->
    {token,{'Start',TokenLine}};
yyaction(2, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    {token,{nt,TokenLine,list_to_atom(TokenChars)}};
yyaction(3, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    {token,{op,TokenLine,list_to_atom(TokenChars)}};
yyaction(4, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    {token,{action,TokenLine,stripa(TokenChars)}};
yyaction(5, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    {token,{cost,TokenLine,stripc(TokenChars)}};
yyaction(6, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    {token,{cost,TokenLine,list_to_integer(TokenChars)}};
yyaction(7, _, _, TokenLine) ->
    {token,{'<-',TokenLine}};
yyaction(8, _, _, TokenLine) ->
    {token,{'(',TokenLine}};
yyaction(9, _, _, TokenLine) ->
    {token,{')',TokenLine}};
yyaction(10, _, _, TokenLine) ->
    {token,{',',TokenLine}};
yyaction(11, _, _, TokenLine) ->
    {token,{'.',TokenLine}};
yyaction(12, _, _, _) ->
    skip_token;
yyaction(13, _, _, _) ->
    skip_token;
yyaction(14, _, _, TokenLine) ->
    {end_token,{'$end',TokenLine}};
yyaction(_, _, _, _) -> error.
