%%%============================================================================
%%% File     : eburg_lexer.xrl
%%% Purpose  : Scanner & tokenizer for eburg specifications.
%%% Author   : j@svirfneblin.org
%%% License  : BSD
%%% Created  : 2007-07-07 10:40:08 j
%%% Modified : 2007-10-12 15:18:52 j
%%%============================================================================

Definitions.
LOWER     = [a-z]
UPPER     = [A-Z]
DIGIT     = [0-9]
CHAR      = ({LOWER}|{UPPER})
WS        = [\000-\s]+

NONTERM   = {LOWER}({LOWER}|{DIGIT}|_)*
OPERATOR  = {UPPER}({UPPER}|{DIGIT}|_)*
DOT       = \.[\n\s\t]
COMMENT   = \%.*
EOF       = \$end

DCOST     = \[:([^]]|[^:]\])*:\]
ACTION    = :([^.]|\.[^\n\s\t])*{DOT}


Rules.
Terminals  : {token, {'Terminals', TokenLine}}.
Start      : {token, {'Start', TokenLine}}.

{NONTERM}  : {token, {nt, TokenLine, list_to_atom(TokenChars)}}.
{OPERATOR} : {token, {op, TokenLine, list_to_atom(TokenChars)}}.

{ACTION}   : {token, {action, TokenLine, stripa(TokenChars)}}.

{DCOST}    : {token, {cost, TokenLine, stripc(TokenChars)}}.
{DIGIT}+   : {token, {cost, TokenLine, list_to_integer(TokenChars)}}.

<-         : {token, {'<-', TokenLine}}.
\(         : {token, {'(',  TokenLine}}.
\)         : {token, {')',  TokenLine}}.
,          : {token, {',',  TokenLine}}.
{DOT}      : {token, {'.',  TokenLine}}.

{WS}       : skip_token.
{COMMENT}  : skip_token.
{EOF}      : {end_token, {'$end', TokenLine}}.


Erlang code.
-import(eburg_util, [butlast/1]).
-export([lex/1]).

%% lex(File: string) -> string
%% lexes `File'. See EOF above.
lex(File) -> string(File ++ "$end").


stripc(Cost)   -> tl(tl(butlast(butlast(Cost)))).
stripa(Action) -> tl(butlast(butlast(Action))).

%%% Local Variables:
%%% mode: erlang
%%% time-stamp-pattern: "Modified : %%$"
%%% End:
